@@ -1,17 +1,18 @@
 /*==========================================================================*/
 /* Runge-Kutta Solvers, (C) Th. Petzoldt, License: GPL >=2                  */
-/*  Euler Fixed Step Integrator                                             */
-/*    (special version with less overhead than the general solution)        */
+/* Euler Fixed Step Integrator                                              */
+/* (special version with less overhead than the general solution)           */
 /*==========================================================================*/
 
 #include "rk_util.h"
+#include "externalptr.h"
 
 SEXP call_euler(SEXP Xstart, SEXP Times, SEXP Func, SEXP Initfunc,
 	        SEXP Parms, SEXP Nout, SEXP Rho, SEXP Verbose,
 		SEXP Rpar, SEXP Ipar, SEXP Flist) {
 
   /* Initialization */
-  long int old_N_Protect = save_N_Protected();
+  int nprot = 0;
 
   double *tt = NULL, *xs = NULL;
   double *tmp, *FF, *out;
@@ -26,11 +27,11 @@ SEXP call_euler(SEXP Xstart, SEXP Times, SEXP Func, SEXP Initfunc,
   /*------------------------------------------------------------------------*/
   /* Processing of Arguments                                                */
   /*------------------------------------------------------------------------*/
-  PROTECT(Times = AS_NUMERIC(Times)); incr_N_Protect();
+  PROTECT(Times = AS_NUMERIC(Times)); nprot++;
   tt = NUMERIC_POINTER(Times);
   nt = length(Times);
 
-  PROTECT(Xstart = AS_NUMERIC(Xstart)); incr_N_Protect();
+  PROTECT(Xstart = AS_NUMERIC(Xstart)); nprot++;
   xs  = NUMERIC_POINTER(Xstart);
   neq = length(Xstart);
 
@@ -45,33 +46,29 @@ SEXP call_euler(SEXP Xstart, SEXP Times, SEXP Func, SEXP Initfunc,
   /*------------------------------------------------------------------------*/
 
   for (i = 0; i < 2; i++) timesteps[i] = tt[1] - tt[0];
-    
+
   /*------------------------------------------------------------------------*/
   /* DLL, ipar, rpar (for compatibility with lsoda)                         */
   /*------------------------------------------------------------------------*/
   int isDll = FALSE;
-  //int ntot  =  0;
   int lrpar= 0, lipar = 0;
   int *ipar = NULL;
 
   if (inherits(Func, "NativeSymbol")) { /* function is a dll */
     isDll = TRUE;
     if (nout > 0) isOut = TRUE;
-    //ntot  = neq + nout;           /* length of yout */
     lrpar = nout + LENGTH(Rpar);  /* length of rpar; LENGTH(Rpar) is always >0 */
     lipar = 3    + LENGTH(Ipar);  /* length of ipar */
 
   } else {                        /* function is not a dll */
     isDll = FALSE;
     isOut = FALSE;
-    //ntot = neq;
     lipar = 3;                    /* in lsoda = 1; */
     lrpar = nout;                 /* in lsoda = 1; */
   }
-  out   = (double *) R_alloc(lrpar, sizeof(double)); 
+  out   = (double *) R_alloc(lrpar, sizeof(double));
   ipar  = (int *) R_alloc(lipar, sizeof(int));
 
-  
   ipar[0] = nout;              /* first 3 elements of ipar are special */
   ipar[1] = lrpar;
   ipar[2] = lipar;
@@ -80,26 +77,26 @@ SEXP call_euler(SEXP Xstart, SEXP Times, SEXP Func, SEXP Initfunc,
     for (j = 0; j < LENGTH(Ipar); j++) ipar[j+3] = INTEGER(Ipar)[j];
     /* out:  first nout elements of out are reserved for output variables
        other elements are set via argument *rpar*  */
-    for (j = 0; j < nout; j++)         out[j] = 0.0;                
+    for (j = 0; j < nout; j++)         out[j] = 0.0;
     for (j = 0; j < LENGTH(Rpar); j++) out[nout+j] = REAL(Rpar)[j];
   }
 
   /*------------------------------------------------------------------------*/
   /* Allocation of Workspace                                                */
   /*------------------------------------------------------------------------*/
-  PROTECT(R_y0 = allocVector(REALSXP, neq)); incr_N_Protect();
-  PROTECT(R_f  = allocVector(REALSXP, neq)); incr_N_Protect();
+  PROTECT(R_y0 = allocVector(REALSXP, neq)); nprot++;
+  PROTECT(R_f  = allocVector(REALSXP, neq)); nprot++;
   y0 = REAL(R_y0);
   f  = REAL(R_f);
 
   /* matrix for holding the outputs */
-  PROTECT(R_yout = allocMatrix(REALSXP, nt, neq + nout + 1)); incr_N_Protect();
+  PROTECT(R_yout = allocMatrix(REALSXP, nt, neq + nout + 1)); nprot++;
   yout = REAL(R_yout);
 
   /* attribute that stores state information, similar to lsoda */
   SEXP R_istate;
   int *istate;
-  PROTECT(R_istate = allocVector(INTSXP, 22)); incr_N_Protect();
+  PROTECT(R_istate = allocVector(INTSXP, 22)); nprot++;
   istate = INTEGER(R_istate);
   istate[0] = 0; /* assume succesful return */
   for (i = 0; i < 22; i++) istate[i] = 0;
@@ -107,7 +104,15 @@ SEXP call_euler(SEXP Xstart, SEXP Times, SEXP Func, SEXP Initfunc,
   /*------------------------------------------------------------------------*/
   /* Initialization of Parameters (for DLL functions)                       */
   /*------------------------------------------------------------------------*/
-  initParms(Initfunc, Parms);
+  if (Initfunc != NA_STRING) {
+    if (inherits(Initfunc, "NativeSymbol")) {
+      init_func_type *initializer;
+      PROTECT(de_gparms = Parms); nprot++;
+      initializer = (init_func_type *) R_ExternalPtrAddrFn_(Initfunc);
+      initializer(Initdeparms);
+    }
+  }
+
   isForcing = initForcings(Flist);
 
   /*------------------------------------------------------------------------*/
@@ -128,7 +133,7 @@ SEXP call_euler(SEXP Xstart, SEXP Times, SEXP Func, SEXP Initfunc,
 
     timesteps[0] = timesteps[1];
     timesteps[1] = dt;
-  
+
     if (verbose)
       Rprintf("Time steps = %d / %d time = %e\n", it + 1, nt, t);
     derivs(Func, t, y0, Parms, Rho, f, out, 0, neq, ipar, isDll, isForcing);
@@ -159,6 +164,6 @@ SEXP call_euler(SEXP Xstart, SEXP Times, SEXP Func, SEXP Initfunc,
 
   timesteps[0] = 0;
   timesteps[1] = 0;
-  restore_N_Protected(old_N_Protect);
+  UNPROTECT(nprot);
   return(R_yout);
 }
